package com.tzld.piaoquan.api.controller.contentplatform;

import com.tzld.piaoquan.api.job.ContentPlatformVideoJob;
import com.tzld.piaoquan.api.model.param.contentplatform.*;
import com.tzld.piaoquan.api.model.vo.contentplatform.GzhPlanItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.QwPlanItemVO;
import com.tzld.piaoquan.api.model.vo.contentplatform.VideoContentItemVO;
import com.tzld.piaoquan.api.service.contentplatform.ContentPlatformPlanService;
import com.tzld.piaoquan.growth.common.common.base.CommonResponse;
import com.tzld.piaoquan.growth.common.utils.page.Page;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/contentPlatform/plan")
@CrossOrigin(origins = "*")
public class ContentPlatformPlanController {

    @Autowired
    ContentPlatformPlanService planService;

    @Autowired
    private ContentPlatformVideoJob videoJob;

    @ApiOperation(value = "公众号计划列表")
    @PostMapping("/gzh/list")
    public CommonResponse<Page<GzhPlanItemVO>> gzhPlanList(@RequestBody GzhPlanListParam param) {
        return CommonResponse.success(planService.gzhPlanList(param));
    }

    @ApiOperation(value = "公众号计划 创建/更新")
    @PostMapping("/gzh/save")
    public CommonResponse<Void> gzhPlanSave(@RequestBody GzhPlanSaveParam param) {
        planService.gzhPlanSave(param);
        return CommonResponse.success();
    }

    @ApiOperation(value = "发布内容视频列表")
    @PostMapping("/videoContentList")
    public CommonResponse<Page<VideoContentItemVO>> getVideoContentList(@RequestBody VideoContentListParam param) {
        return CommonResponse.success(planService.getVideoContentList(param));
    }

    @ApiOperation(value = "发布内容视频品类列表")
    @GetMapping("/videoContentCategoryList")
    public CommonResponse<List<String>> getVideoContentCategoryList() {
        return CommonResponse.success(planService.getVideoContentCategoryList());
    }

    @ApiOperation(value = "发布内容视频封面截帧列表")
    @PostMapping("/videoContentCoverFrameList")
    public CommonResponse<List<String>> getVideoContentCoverFrameList(@RequestBody VideoContentCoverFrameParam param) {
        return CommonResponse.success(planService.getVideoContentCoverFrameList(param));
    }

    @ApiOperation(value = "企微计划列表")
    @PostMapping("/qw/list")
    public CommonResponse<Page<QwPlanItemVO>> qwPlanList(@RequestBody QwPlanListParam param) {
        return CommonResponse.success(planService.qwPlanList(param));
    }

    @ApiOperation(value = "企微计划 创建/更新")
    @PostMapping("/qw/save")
    public CommonResponse<Void> qwPlanSave(@RequestBody QwPlanSaveParam param) {
        planService.qwPlanSave(param);
        return CommonResponse.success();
    }

    @ApiOperation(value = "同步每日top1000视频")
    @PostMapping("/syncContentPlatformVideoJob")
    public CommonResponse<Void> syncContentPlatformVideoJob(@RequestBody QwPlanSaveParam param) {
        videoJob.syncContentPlatformVideoJob(null);
        return CommonResponse.success();
    }
}
